import django
from django.test import Client, TestCase

django.setup()

try:
    from unittest.mock import MagicMock
except ImportError:
    from mock import MagicMock


class RequestIDTestCase(TestCase):
    def setUp(self):
        self.c = Client()

    def test_middleware(self):
        response = self.c.get("/index/")

        self.assertTrue(response.has_header("HTTP_X_REQUEST_ID"))



import uuid

from django.conf import settings

from request_id_django_log import local_threading
from request_id_django_log.settings import NO_REQUEST_ID

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object


class RequestIdDjangoLog(MiddlewareMixin):
    def process_request(self, request):
        configs = getattr(settings, "REQUEST_ID_CONFIG", None)
        if configs:
            request_id = self._get_request_id(request, configs)
            request.id = request_id
            local_threading.request_id = request_id

    def process_response(self, request, response):
        configs = getattr(settings, "REQUEST_ID_CONFIG", None)
        if not configs:
            return response

        if configs.get("RESPONSE_HEADER_REQUEST_ID", False):
            response[configs.get("RESPONSE_HEADER_REQUEST_ID")] = request.id

        return response

    def _get_request_id(self, request, configs):
        header = configs.get("REQUEST_ID_HEADER", None)

        generate_request_if_not_found = configs.get(
            "GENERATE_REQUEST_ID_IF_NOT_FOUND", False
        )

        if header:
            default = NO_REQUEST_ID

            if generate_request_if_not_found:
                default = self._generate_id()

            return request.META.get(header, default)

        return self._generate_id()

    @staticmethod
    def _generate_id():
        return str(uuid.uuid4())



DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": "mydatabase"}}
DEBUG = True
SECRET_KEY = "test"
ALLOWED_HOSTS = ["*"]
REQUEST_ID_CONFIG = {
    "REQUEST_ID_HEADER": "HTTP_X_REQUEST_ID",
    "GENERATE_REQUEST_ID_IF_NOT_FOUND": True,
    "RESPONSE_HEADER_REQUEST_ID": "HTTP_X_REQUEST_ID",
}

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "request_id_django_log",
]

MIDDLEWARE = [
    "request_id_django_log.middleware.RequestIdDjangoLog",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "tests.urls"

MIDDLEWARE_CLASSES = [
    "request_id_django_log.middleware.RequestIdDjangoLog",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]



import uuid

from django.test import TestCase

from request_id_django_log import local_threading
from request_id_django_log.apps import RequestIdDjangoLogConfig
from request_id_django_log.filters import RequestIDFilter
from request_id_django_log.middleware import RequestIdDjangoLog
from request_id_django_log.request_id import current_request_id

try:
    from unittest.mock import MagicMock, patch
except ImportError:
    from mock import MagicMock, patch


class RequestIDTestCase(TestCase):
    def setUp(self):
        pass

    def test_app(self):
        "APP: Shoud return app name request_id_django_log when RequestIdDjangoLogConfig exists"
        self.assertEqual(RequestIdDjangoLogConfig.name, "request_id_django_log")

    def test_filters_should_return_none(self):
        """Filter: Should return none when local_threading dont have request_id"""
        mock_record = MagicMock()
        filter = RequestIDFilter(MagicMock())
        result = filter.filter(mock_record)

        self.assertEqual(mock_record.request_id, "none")
        self.assertTrue(result)
        self.assertIsInstance(filter, RequestIDFilter)

    def test_filters_should_return_id(self):
        """Filter: Should return request_id when local_threading have request_id"""
        local_threading.request_id = "12345"
        mock_record = MagicMock()
        filter = RequestIDFilter(MagicMock())

        result = filter.filter(mock_record)
        del local_threading.request_id

        self.assertEqual(mock_record.request_id, "12345")
        self.assertTrue(result)
        self.assertIsInstance(filter, RequestIDFilter)

    def test_middleware_get_request_id_return_id_from_header(self):
        """Middleware: Should return request_id equals header when header request_id is present"""
        mock_request = MagicMock()
        mock_request.META = {"request_is": "123"}

        mid = RequestIdDjangoLog()
        request_id = mid._get_request_id(
            mock_request, {"REQUEST_ID_HEADER": "request_is"}
        )

        self.assertEqual(request_id, "123")

    def test_middleware_get_request_id_generate_id_header_not_found(self):
        """Middleware: Should return generated request_id when config header not found"""
        mock_request = MagicMock()

        mid = RequestIdDjangoLog()
        request_id = mid._get_request_id(mock_request, {})

        self.assertTrue(uuid.UUID(request_id))

    def test_middleware_get_request_id_generate_id_request_header_not_found(self):
        """Middleware: Should return request_id equals header when header request_id is present"""
        mock_request = MagicMock()
        mock_request.META = {}

        configs = {
            "REQUEST_ID_HEADER": "request_is",
            "GENERATE_REQUEST_ID_IF_NOT_FOUND": True,
        }

        mid = RequestIdDjangoLog()
        request_id = mid._get_request_id(mock_request, configs)

        self.assertTrue(uuid.UUID(request_id))

    def test_middleware_get_request_id_none_when_not_need_generate(self):
        """Middleware: Should return request_id none when generate config not found"""
        mock_request = MagicMock()
        mock_request.META = {}
        mid = RequestIdDjangoLog()
        request_id = mid._get_request_id(
            mock_request, {"REQUEST_ID_HEADER": "request_is"}
        )

        self.assertEqual(request_id, "none")

    def test_local_threading_return_id(self):
        """Current Request ID: Should return 123 when local_thead have value"""

        local_threading.request_id = "123"
        self.assertEqual(current_request_id(), "123")
        del local_threading.request_id

    def test_local_threading_return_none(self):
        """Current Request ID: Should return none when local_thead dont have value"""

        self.assertEqual(current_request_id(), "none")



from request_id_django_log import local_threading
from request_id_django_log.settings import NO_REQUEST_ID


def current_request_id():
    try:
        return local_threading.request_id
    except AttributeError:
        return NO_REQUEST_ID



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="request_id_django_log",
    version="0.1.1",
    description="A simple way to implement request_id in Django",
    long_description=README,
    long_description_content_type='text/markdown',
    include_package_data=True,
    author="Ricardo Baltazar Chaves",
    author_email="ricardobchaves6@gmail.com",
    license="MIT",
    url="https://github.com/juntossomosmais/request-id-django-log",
    packages=find_packages(),
    install_requires=[],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 1.8",
        "Framework :: Django :: 1.9",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Environment :: Web Environment",
        "Natural Language :: Portuguese (Brazilian)",
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import logging

from request_id_django_log import local_threading
from request_id_django_log.settings import NO_REQUEST_ID


class RequestIDFilter(logging.Filter):
    def filter(self, record):
        record.request_id = getattr(local_threading, "request_id", NO_REQUEST_ID)
        return True
